import React, { useState, useEffect, lazy, Suspense } from 'react';
import { BrowserRouter as Router, Routes, Route } from 'react-router-dom';
import { Upload, CheckCircle, AlertTriangle, Info, Shield, FileText, ArrowRight, Link as LinkIcon, Loader } from 'lucide-react';
import Header from './components/Header';
import Footer from './components/Footer';
import { ToastProvider } from './components/ui/Toast';
import AnalysisHistory from './components/ui/AnalysisHistory';
import { useAnalysisStore } from './store/analysisStore';
import RegulationsPage from './pages/RegulationsPage';
import GDPRPage from './pages/regulations/GDPRPage';
import CCPAPage from './pages/regulations/CCPAPage';
import StateLawsPage from './pages/regulations/StateLawsPage';
import InternationalPage from './pages/regulations/InternationalPage';
import ResourcesPage from './pages/ResourcesPage';
import PrivacyGuidePage from './pages/resources/PrivacyGuidePage';
import ComplianceChecklistPage from './pages/resources/ComplianceChecklistPage';
import FAQPage from './pages/resources/FAQPage';
import PrivacyPolicyExamples from './pages/resources/PrivacyPolicyExamples';
import PlainLanguagePrivacyPolicy from './pages/resources/privacy-examples/PlainLanguagePrivacyPolicy';
import EcommercePrivacyPolicy from './pages/resources/privacy-examples/EcommercePrivacyPolicy';
import BlogPage from './pages/BlogPage';
import BlogPostPage from './pages/BlogPostPage';
import PlainLanguagePrivacyBlogPost from './pages/blog/PlainLanguagePrivacyBlogPost';
import LLMPrivacyBlogPost from './pages/blog/LLMPrivacyBlogPost';
import ContactPage from './pages/ContactPage';
import PrivacyLLMPage from './pages/PrivacyLLMPage';
import ErrorBoundary from './components/ErrorBoundary';
import LoadingSpinner from './components/LoadingSpinner';
import AIPrivacyRegulationsMay2025 from './pages/blog/AIPrivacyRegulationsMay2025';
import AIPrivacyJune2025 from './pages/blog/AIPrivacyJune2025';
import { PolicyAnalysisResult } from './types';
import UrlImporter from './components/UrlImporter';
import { extractTextFromFile } from './utils/fileHandlers';
import FeedbackWidget from './components/FeedbackWidget';
import { showToast } from './components/ui/Toast';
import KeyboardShortcuts from './components/ui/KeyboardShortcuts';
import { useKeyboardShortcuts } from './hooks/useKeyboardShortcuts';

// Lazy load components
const PolicyAuditor = lazy(() => import('./components/PolicyAuditor'));
const ComplianceReport = lazy(() => import('./components/ComplianceReport'));

function HomePage() {
  const [policyText, setPolicyText] = useState<string>('');
  const [isAnalyzing, setIsAnalyzing] = useState<boolean>(false);
  const [isImportingUrl, setIsImportingUrl] = useState<boolean>(false);
  const [fileError, setFileError] = useState<string | null>(null);
  const [isProcessingFile, setIsProcessingFile] = useState<boolean>(false);
  
  const { 
    currentAnalysis, 
    setCurrentAnalysis, 
    addToHistory, 
    setAnalyzing, 
    setError 
  } = useAnalysisStore();

  // Keyboard shortcuts
  useKeyboardShortcuts([
    {
      key: 'n',
      ctrlKey: true,
      callback: () => {
        setPolicyText('');
        setCurrentAnalysis(null);
        setFileError(null);
        showToast.info('Ready for new analysis');
      },
      description: 'New analysis'
    },
    {
      key: 's',
      ctrlKey: true,
      callback: () => {
        if (currentAnalysis) {
          // Trigger download
          showToast.info('Downloading current analysis...');
        } else {
          showToast.error('No analysis to save');
        }
      },
      description: 'Save current analysis'
    }
  ]);

  const handleTextChange = (e: React.ChangeEvent<HTMLTextAreaElement>) => {
    setPolicyText(e.target.value);
    setCurrentAnalysis(null);
    setFileError(null);
  };

  const handleFileUpload = async (e: React.ChangeEvent<HTMLInputElement>) => {
    const file = e.target.files?.[0];
    if (!file) return;

    setIsProcessingFile(true);
    setFileError(null);

    try {
      const text = await extractTextFromFile(file);
      setPolicyText(text);
      setCurrentAnalysis(null);
      showToast.success('File processed successfully');
    } catch (error) {
      console.error('Error processing file:', error);
      setFileError(error instanceof Error ? error.message : 'Failed to process the file');
      showToast.error('Failed to process file');
    } finally {
      setIsProcessingFile(false);
    }
  };

  const handleUrlImport = (url: string, content: string) => {
    setPolicyText(content);
    setCurrentAnalysis(null);
    setFileError(null);
    setIsImportingUrl(false);
    showToast.success('Content imported successfully');
  };

  const analyzePolicy = async () => {
    if (!policyText.trim()) return;
    
    setIsAnalyzing(true);
    setAnalyzing(true);
    setFileError(null);
    setError(null);
    
    try {
      const worker = new Worker(new URL('./utils/policyAnalyzer.worker.ts', import.meta.url), { type: 'module' });
      
      worker.onmessage = function(e) {
        if (e.data.success) {
          setCurrentAnalysis(e.data.result);
          addToHistory(e.data.result, policyText, `Analysis ${new Date().toLocaleDateString()}`);
          showToast.success('Analysis completed successfully');
        } else {
          console.error('Worker error:', e.data.error);
          setFileError('Error analyzing policy. Please try again.');
          setError(e.data.error);
          showToast.error('Analysis failed');
        }
        setIsAnalyzing(false);
        setAnalyzing(false);
        worker.terminate();
      };
      
      worker.onerror = function(error) {
        console.error('Worker error:', error);
        setFileError('Error analyzing policy. Please try again.');
        setError('Worker error occurred');
        showToast.error('Analysis failed');
        setIsAnalyzing(false);
        setAnalyzing(false);
        worker.terminate();
      };
      
      worker.postMessage(policyText);
    } catch (error) {
      console.error('Error analyzing policy:', error);
      setFileError('Failed to analyze policy. Please try again.');
      setError('Failed to analyze policy');
      showToast.error('Analysis failed');
      setIsAnalyzing(false);
      setAnalyzing(false);
    }
  };

  return (
    <div className="min-h-screen bg-gradient-to-b from-gray-50 to-gray-100 flex flex-col">
      <main className="flex-grow container mx-auto px-4 py-8">
        <div className="max-w-4xl mx-auto">
          <ErrorBoundary>
            <section className="mb-10">
              <AnalysisHistory />
              
              <div className="bg-white rounded-lg shadow-md p-6 mb-8">
                <h2 className="text-2xl font-bold text-gray-800 mb-4 flex items-center">
                  <Shield className="mr-2 text-indigo-600" size={24} />
                  Privacy Policy Compliance Auditor
                </h2>
                
                <p className="text-gray-600 mb-6">
                  Analyze your privacy policy for compliance with GDPR, CCPA, and other privacy regulations. 
                  Enter your privacy policy text below, upload a file, or import from a URL to get started.
                </p>
                
                <div className="mb-6">
                  <label htmlFor="policyText" className="block text-sm font-medium text-gray-700 mb-2">
                    Privacy Policy Text
                  </label>
                  <textarea
                    id="policyText"
                    className="w-full h-64 p-3 border border-gray-300 rounded-md focus:ring-2 focus:ring-indigo-500 focus:border-indigo-500"
                    placeholder="Paste your privacy policy text here..."
                    value={policyText}
                    onChange={handleTextChange}
                  ></textarea>
                </div>
                
                {fileError && (
                  <div className="mb-4 p-3 bg-red-50 border border-red-200 rounded-md">
                    <div className="flex items-start">
                      <AlertTriangle className="text-red-500 mt-0.5 mr-2" size={16} />
                      <p className="text-sm text-red-700">{fileError}</p>
                    </div>
                  </div>
                )}
                
                <div className="flex flex-col sm:flex-row gap-4 mb-6">
                  <div className="relative">
                    <input
                      type="file"
                      id="fileUpload"
                      className="sr-only"
                      accept=".txt,.md,.html,.htm,.docx,.pdf"
                      onChange={handleFileUpload}
                      disabled={isProcessingFile}
                    />
                    <label
                      htmlFor="fileUpload"
                      className={`flex items-center justify-center px-4 py-2 border border-gray-300 rounded-md shadow-sm text-sm font-medium text-gray-700 bg-white hover:bg-gray-50 ${isProcessingFile ? 'cursor-wait opacity-70' : 'cursor-pointer'}`}
                    >
                      {isProcessingFile ? (
                        <>
                          <LoadingSpinner size="small" className="mr-2" />
                          Processing...
                        </>
                      ) : (
                        <>
                          <Upload className="mr-2" size={16} />
                          Upload Policy File
                        </>
                      )}
                    </label>
                  </div>
                  
                  <button
                    onClick={() => setIsImportingUrl(true)}
                    className="flex items-center justify-center px-4 py-2 border border-gray-300 rounded-md shadow-sm text-sm font-medium text-gray-700 bg-white hover:bg-gray-50"
                    disabled={isProcessingFile}
                  >
                    <LinkIcon className="mr-2" size={16} />
                    Import from URL
                  </button>
                  
                  <button
                    onClick={analyzePolicy}
                    disabled={!policyText.trim() || isAnalyzing || isProcessingFile}
                    className={`flex items-center justify-center px-4 py-2 border border-transparent rounded-md shadow-sm text-sm font-medium text-white ${
                      !policyText.trim() || isAnalyzing || isProcessingFile
                        ? 'bg-indigo-300 cursor-not-allowed'
                        : 'bg-indigo-600 hover:bg-indigo-700'
                    }`}
                  >
                    {isAnalyzing ? (
                      <>
                        <LoadingSpinner size="small" className="mr-2" />
                        Analyzing...
                      </>
                    ) : (
                      <>
                        <ArrowRight className="mr-2" size={16} />
                        Analyze Policy
                      </>
                    )}
                  </button>
                </div>
                
                <div className="flex flex-wrap gap-3 text-sm text-gray-600">
                  <div className="flex items-center">
                    <CheckCircle className="text-green-500 mr-1" size={16} />
                    <span>GDPR</span>
                  </div>
                  <div className="flex items-center">
                    <CheckCircle className="text-green-500 mr-1" size={16} />
                    <span>CCPA</span>
                  </div>
                  <div className="flex items-center">
                    <CheckCircle className="text-green-500 mr-1" size={16} />
                    <span>CPRA</span>
                  </div>
                  <div className="flex items-center">
                    <CheckCircle className="text-green-500 mr-1" size={16} />
                    <span>VCDPA</span>
                  </div>
                  <div className="flex items-center">
                    <CheckCircle className="text-green-500 mr-1" size={16} />
                    <span>CPA</span>
                  </div>
                  <div className="flex items-center">
                    <CheckCircle className="text-green-500 mr-1" size={16} />
                    <span>CTDPA</span>
                  </div>
                </div>
              </div>
              
              <Suspense fallback={<LoadingSpinner size="large" />}>
                {currentAnalysis && <ComplianceReport result={currentAnalysis} />}
              </Suspense>
            </section>
          </ErrorBoundary>
        </div>
      </main>
      
      {isImportingUrl && (
        <UrlImporter 
          onImport={handleUrlImport} 
          onCancel={() => setIsImportingUrl(false)} 
        />
      )}
    </div>
  );
}

function App() {
  return (
    <Router>
      <div className="min-h-screen bg-gradient-to-b from-gray-50 to-gray-100 flex flex-col">
        <Header />
        <ErrorBoundary>
          <Suspense fallback={<LoadingSpinner size="large" />}>
            <Routes>
              <Route path="/" element={<HomePage />} />
              <Route path="/regulations" element={<RegulationsPage />} />
              <Route path="/regulations/gdpr" element={<GDPRPage />} />
              <Route path="/regulations/ccpa" element={<CCPAPage />} />
              <Route path="/regulations/state-laws" element={<StateLawsPage />} />
              <Route path="/regulations/international" element={<InternationalPage />} />
              <Route path="/resources" element={<ResourcesPage />} />
              <Route path="/resources/guide" element={<PrivacyGuidePage />} />
              <Route path="/resources/checklist" element={<ComplianceChecklistPage />} />
              <Route path="/resources/faq" element={<FAQPage />} />
              <Route path="/resources/privacy-policy-examples" element={<PrivacyPolicyExamples />} />
              <Route path="/resources/privacy-policy-examples/plain-language" element={<PlainLanguagePrivacyPolicy />} />
              <Route path="/resources/privacy-policy-examples/ecommerce" element={<EcommercePrivacyPolicy />} />
              <Route path="/blog" element={<BlogPage />} />
              <Route path="/blog/:id" element={<BlogPostPage />} />
              <Route path="/blog/plain-language-privacy-2025" element={<PlainLanguagePrivacyBlogPost />} />
              <Route path="/blog/llm-privacy-2025" element={<LLMPrivacyBlogPost />} />
              <Route path="/blog/ai-privacy-regulations-may-2025" element={<AIPrivacyRegulationsMay2025 />} />
              <Route path="/blog/ai-privacy-june-2025" element={<AIPrivacyJune2025 />} />
              <Route path="/contact" element={<ContactPage />} />
              <Route path="/pa-llm" element={<PrivacyLLMPage />} />
            </Routes>
          </Suspense>
        </ErrorBoundary>
        
        <Footer />
        <FeedbackWidget />
        <ToastProvider />
        <KeyboardShortcuts />
      </div>
    </Router>
  );
}

export default App;